const consumerKey = process.env.TWITTER_CLIENT_ID
const consumerSecret = process.env.TWITTER_CLIENT_SECRET
import logger from 'logger/logger'
import { OAuth } from 'oauth'
import { promisify } from 'util'

export async function exchangeTwitterCodeForToken(code: string, vcode: string) {
  const url = 'https://api.twitter.com/2/oauth2/token'
  const credentials = Buffer.from(`${consumerKey}:${consumerSecret}`).toString('base64')
  const response = await fetch(url, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/x-www-form-urlencoded',
      Authorization: `Basic ${credentials}`,
    },
    body: new URLSearchParams({
      code,
      grant_type: 'authorization_code',
      // client_id: process.env.TWITTER_CLIENT_ID,
      redirect_uri: 'https://oauth-svr.cebggame.com/twitter/redirect_uri',
      code_verifier: vcode,
    }),
  })
  const data = await response.json()
  return data
}

/**
 * https://developer.twitter.com/en/docs/authentication/oauth-2-0/authorization-code
 * @param refreshToken
 * @returns
 */
export async function refreshToken(refreshToken: string) {
  const url = 'https://api.twitter.com/2/oauth2/token'
  const credentials = Buffer.from(`${consumerKey}:${consumerSecret}`).toString('base64')
  const response = await fetch(url, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/x-www-form-urlencoded',
      Authorization: `Basic ${credentials}`,
    },
    body: new URLSearchParams({
      grant_type: 'refresh_token',
      refresh_token: refreshToken,
    }),
  })
  const data = await response.json()
  return data
}

/**
 * https://developer.twitter.com/en/docs/twitter-api/users/lookup/api-reference/get-users-me
 * User rate limit (User context): 75 requests per 15-minute window per each authenticated user
 * @param accessToken
 * @returns
 */
export async function getTwitterUserInfo(accessToken: string) {
  const url = 'https://api.twitter.com/2/users/me?user.fields=profile_image_url'
  const response = await fetch(url, {
    method: 'GET',
    headers: {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${accessToken}`,
    },
  })
  const data = await response.json()
  return data
}

var oauthConsumer = new OAuth(
  'https://api.twitter.com/oauth/request_token',
  'https://api.twitter.com/oauth/access_token',
  process.env.TWITTER_API_KEY,
  process.env.TWITTER_API_SECRET,
  '1.0A',
  process.env.TWITTER_OAUTH1_CB,
  'HMAC-SHA1',
)

export async function getOAuthAccessTokenWith({
  oauthRequestToken,
  oauthRequestTokenSecret,
  oauthVerifier,
}: any = {}): Promise<any> {
  return new Promise((resolve, reject) => {
    oauthConsumer.getOAuthAccessToken(
      oauthRequestToken,
      oauthRequestTokenSecret,
      oauthVerifier,
      function (error, oauthAccessToken, oauthAccessTokenSecret, results) {
        if (error) {
          logger.error(error)
          reject(new Error('Error getting OAuth access token'))
        } else {
          resolve({ oauthAccessToken, oauthAccessTokenSecret, results })
        }
      },
    )
  })
}
export async function getOAuthRequestToken() {
  return new Promise((resolve, reject) => {
    oauthConsumer.getOAuthRequestToken(function (error, oauthRequestToken, oauthRequestTokenSecret, results) {
      if (error) {
        logger.error(error)
        reject(new Error('Error getting OAuth request token'))
      } else {
        resolve({ oauthRequestToken, oauthRequestTokenSecret, results })
      }
    })
  })
}

export async function oauthGetUserById(userId, { oauthAccessToken, oauthAccessTokenSecret }: any = {}) {
  return promisify(oauthConsumer.get.bind(oauthConsumer))(
    `https://api.twitter.com/1.1/users/show.json?user_id=${userId}`,
    oauthAccessToken,
    oauthAccessTokenSecret,
  ).then(body => JSON.parse(body))
}
