/**
 * 根据key升序生成 key1=val1&key2=val2的字符串
 * @param {object} data 需要处理的对象
 * @param {boolean} ignoreNull 是否过滤空值(空格或者null值不参与拼接)
 * @param splitChar 连接的字符, 默认是&
 * @param equalChar =
 */
export function generateKeyValStr(data: {}, ignoreNull = true, splitChar: string = '&', equalChar = '=') {
  const keys = Object.keys(data)
  keys.sort()
  let result = ''
  let i = 0
  for (let key of keys) {
    if (ignoreNull && !data[key]) {
      return
    }
    if (i++ > 0) result += splitChar
      result += `${key}${equalChar}${data[key]}`
  }
  return result
}

/**
 * 将key1=val&key2=val的字符串组装成对象
 * @param str key1=val&key2=val的字符串
 * @param splitChar 连接的字符, 默认是&
 * @param equalChar =
 */
export function keyValToObject(str: string, splitChar: string = '&', equalChar = '='): {} {
  let result = {}
  if (!str) {
    return result
  }
  let arrs = str.split(splitChar)
  for (let sub of arrs) {
    let subArr = sub.split(equalChar)
    result[subArr[0]] = subArr[1]
  }
  return result
}

/**
 * 判断传入的值是否为true
 * @param {Object} obj   传入值为'true','TRUE',1,'1','on','ON','YES','yes'时,返回true,其他值均返回false
 * @return {boolean}
 */
export function isTrue(obj) {
  return (
    obj === 'true' ||
      obj === 'TRUE' ||
      obj === 'True' ||
      obj === 'on' ||
      obj === 'ON' ||
      obj === true ||
      obj === 1 ||
      obj === '1' ||
      obj === 'YES' ||
      obj === 'yes'
  )
}

/**
 * 验证ObjectId格式是否正确
 * @param {string} id
 * @return {boolean}
 */
export function isObjectId(id: string): boolean {
  //mongoose.Types.ObjectId.isValid(id)
  return /^[a-fA-F0-9]{24}$/.test(id)
}

/**
 * 10进制 -> 62进制
 * @param {string | number} number
 * @return {string}
 */
export function string10to62(number: string | number) {
  const chars = '0123456789abcdefghigklmnopqrstuvwxyzABCDEFGHIGKLMNOPQRSTUVWXYZ'.split('')
  const radix = chars.length
  let qutient = +number
  const arr = []
  do {
    const mod = qutient % radix
    qutient = (qutient - mod) / radix
    arr.unshift(chars[mod])
  } while (qutient)
    return arr.join('')
}

/**
 * 62进制 -> 10 进制
 * @param {string} numberCode
 * @return {number}
 */
export function string62to10(numberCode: string) {
  const chars = '0123456789abcdefghigklmnopqrstuvwxyzABCDEFGHIGKLMNOPQRSTUVWXYZ'
  const radix = chars.length
  numberCode = numberCode + ''
  const len = numberCode.length
  let i = 0
  let originNumber = 0
  while (i < len) {
    originNumber += Math.pow(radix, i++) * (chars.indexOf(numberCode.charAt(len - i)) || 0)
  }
  return originNumber
}

const reNormalUUID = /^[0-9a-fA-F-]{36}$/;
const reLongUUID = /^[0-9a-fA-F]{32}$/;
const reShortUUID = /^[0-9a-zA-Z+/]{22,23}$/;
const n = /-/g;

export function compressUuid(e:string, t: boolean = false) {
  if (reNormalUUID.test(e)) {
    e = e.replace(n, '');
  } else if (!reLongUUID.test(e)) {
    return e;
  }
  var r = !0 === t ? 2 : 5;
  return compressHex(e, r)
} 

const CHARS_BASE64 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
export function compressHex(e: string, r: number) {
  var i, n = e.length;
  i = void 0 !== r ? r : n % 3;
  for (var s = e.slice(0, i), o = []; i < n;) {
    var u = parseInt(e[i], 16),
      a = parseInt(e[i + 1], 16),
      c = parseInt(e[i + 2], 16);
    o.push(CHARS_BASE64[u << 2 | a >> 2]);
    o.push(CHARS_BASE64[(3 & a) << 4 | c]);
    i += 3;
  }
  return s + o.join('')
}
